package de.fraunhofer.iem.dataprovider.repository.service

import de.fraunhofer.iem.dataprovider.gitlab.service.OpenCodeGitlabApi
import de.fraunhofer.iem.dataprovider.logger.getLogger
import de.fraunhofer.iem.dataprovider.repository.dto.RepositoryCreateDto
import de.fraunhofer.iem.dataprovider.repository.entity.RepositoryEntity
import de.fraunhofer.iem.dataprovider.repository.repository.RepositoryRepository
import org.springframework.stereotype.Service
import java.util.*

@Service
class RepositoryService(
    private val repositoryRepository: RepositoryRepository,
    private val openCodeGitlabApi: OpenCodeGitlabApi
) {
    private val logger = getLogger(javaClass)

    /**
     * Either creates or returns a repository entity based upon its
     * opencode repository id.
     */
    fun getOrCreate(gitRepository: RepositoryCreateDto): RepositoryEntity {
        var repo = repositoryRepository.findByRepoId(gitRepository.id)
        if (repo == null) {
            repo = repositoryRepository.save(gitRepository.toDbObject())
        }
        return repo
    }

    /**
     * Queries the gitlab api to get the repo name, url, and id.
     */
    fun getRepositoryInfo(repoId: Long): RepositoryCreateDto {
        return openCodeGitlabApi.getRepositoryInfo(repoId)
    }

    // TODO: We should decide if we want to return optionals (null if not found) or throw exceptions or ...
    // TODO: Must be consistent across all methods
    fun findRepoByID(id: UUID): RepositoryEntity? {
        val repository = repositoryRepository.findById(id)
        if (repository.isEmpty) {
            return null
        }
        return repository.get()
    }

    fun save(repo: RepositoryEntity) {
        repositoryRepository.save(repo)
    }

    fun getAllRepositories(): List<RepositoryEntity> {
        return repositoryRepository.findAll()
    }
}
